import characterRepository from '#repositories/characterRepository'
import { BaseEvent } from '#application/base/baseEvent'
import TileRepository from '#repositories/tileRepository'
import { UUID } from '#application/types'

type Payload = {
  id: UUID
  name: string
  tags: string[]
}

export default class TileUpdateEvent extends BaseEvent{
  public listen(): void {
    this.socket.on('gm:tile:update', this.handleEvent.bind(this))
  }

  private async handleEvent(data: Payload, callback: (success: boolean) => void): Promise<void> {
    const character = await characterRepository.getById(this.socket.characterId!)
    if (!character) return callback(false)

    if (character.role !== 'gm') {
      return
    }

    try {
      const tile = await TileRepository.getById(data.id)
      if (!tile) return callback(false)

      await tile.setName(data.name).setTags(data.tags).update()

      callback(true)
    } catch (error) {
      console.error(error)
      callback(false)
    }
  }
}
