import fs from 'fs/promises'
import { writeFile } from 'node:fs/promises'

import Storage from '#application/storage'
import characterRepository from '#repositories/characterRepository'
import { BaseEvent } from '#application/base/baseEvent'
import { Tile } from '#entities/tile'

interface ITileData {
  [key: string]: Buffer
}

export default class TileUploadEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:tile:upload', this.handleEvent.bind(this))
  }

  private async handleEvent(data: ITileData, callback: (response: boolean) => void): Promise<void> {
    try {
      const character = await characterRepository.getById(this.socket.characterId!)
      if (!character) return callback(false)

      if (character.role !== 'gm') {
        return
      }

      const public_folder = Storage.getPublicPath('tiles')

      // Ensure the folder exists
      await fs.mkdir(public_folder, { recursive: true })

      const uploadPromises = Object.entries(data).map(async ([key, tileData]) => {
        const tile = new Tile()
        await tile.setName('New tile').save()
        const uuid = tile.getId()
        const filename = `${uuid}.png`
        const finalFilePath = Storage.getPublicPath('tiles', filename)
        await writeFile(finalFilePath, tileData)
      })

      await Promise.all(uploadPromises)

      callback(true)
    } catch (error) {
      this.logger.error('Error uploading tile:', error)
      callback(false)
    }
  }
}
